/*
 * ChangeElementAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

#include "Helpers/MemDebug.hpp"

#include "Actions/AtomAction/ChangeElementAction.hpp"
#include "atom.hpp"
#include "log.hpp"
#include "vector.hpp"
#include "verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "UIElements/ValueStorage.hpp"

const char AtomChangeElementAction::NAME[] = "change-element";

AtomChangeElementAction::AtomChangeElementAction() :
  Action(NAME)
{}

AtomChangeElementAction::~AtomChangeElementAction()
{}

Dialog* AtomChangeElementAction::createDialog() {
  Dialog *dialog = UIFactory::getInstance().makeDialog();

  dialog->queryElement(NAME, ValueStorage::getInstance().getDescription(NAME));

return dialog;
}

Action::state_ptr AtomChangeElementAction::performCall() {
  atom *first = NULL;
  std::vector<element *> elements;

  ValueStorage::getInstance().queryCurrentValue(NAME, elements);

  ASSERT(elements.size() == 1, "Unequal to one element specified when changing an atom's element");
  if (elements.at(0) == NULL) {
    return Action::failure;
  }
  for (World::AtomSelectionIterator iter = World::getInstance().beginAtomSelection(); iter != World::getInstance().endAtomSelection(); ++iter) {
    first = iter->second;
    DoLog(1) && (Log() << Verbose(1) << "Changing atom " << *first << " to element " << elements.at(0) << "." << endl);
    first->type = elements.at(0);
  }
  return Action::success;
}

Action::state_ptr AtomChangeElementAction::performUndo(Action::state_ptr _state) {
//  ParserLoadXyzState *state = assert_cast<ParserLoadXyzState*>(_state.get());

  return Action::failure;
//  string newName = state->mol->getName();
//  state->mol->setName(state->lastName);
//
//  return Action::state_ptr(new ParserLoadXyzState(state->mol,newName));
}

Action::state_ptr AtomChangeElementAction::performRedo(Action::state_ptr _state){
  return Action::failure;
}

bool AtomChangeElementAction::canUndo() {
  return false;
}

bool AtomChangeElementAction::shouldUndo() {
  return false;
}

const string AtomChangeElementAction::getName() {
  return NAME;
}
